


------------------------------------
---    data type define
------------------------------------

-- enum
PlayOrder = {
    Simultaneously  = 0,
    Randomly        = 1,
    Sequentially    = 2,
    PickOneRandomly = 3
}

AnimationAction = {
    Disappear       = 0,
    Appear          = 1
}

ValidPeriod = {
    Indefinite      = 0,
    DefineTime      = 1,
    DefineCount     = 2
}

TrigCondition = {
    And             = 0,
    Or              = 1
}

TrigType = {
    None            = 0,
    Face            = 1,
    Hand            = 2,
}

TrigSubTypeFace = {
    -- trigger by face
    None                = 0,
    CloseLeftEye        = 1,
    CloseRightEye       = 2,
    RaiseLeftEyebrow    = 3,
    RaiseRightEyebrow   = 4,
    MouthOpen           = 5,
    Pout                = 6,
    Smile               = 7,
    Sad                 = 8,
}

TrigSubTypeHand = {
    -- trigger by hand
    Appear          = 0,
    Disappear       = 1
}



-- NodeEffectElement
NodeEffectElement = {

    type = "",
    name = "",
    --- animation
    animationAction = AnimationAction.Appear,

    --- play
    startFrame = 0,
    endFrame = 1,
    validPeriod = ValidPeriod.Indefinite,
    validPeriodParam = 0,

    --- after play
    stopAtLastFrame = true,
    keepForever = true ,
    keepDuration = 1  -- seconds
}

function NodeEffectElement:new (o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end


-- NodeEffectModel
NodeEffectModel = {
    eventType = 1,
    playOrder = PlayOrder.Simultaneously,
    effectElements = {}  -- array of NodeEffectElement
}
function NodeEffectModel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end


-- NodeTriggerElement
NodeTriggerElement = {
    name = "",
    trigType = TrigType.Face,
    trigSubType = TrigSubTypeFace.CloseLeftEye
}
function NodeTriggerElement:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- NodeTriggerGroup
NodeTriggerGroup = {
    eventType = 2,
    trigCondition = TrigCondition.Or,
    portId = "",
    triggerElements = {}  -- array of NodeTriggerElement
}
function NodeTriggerGroup:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- NodeTriggerModel
NodeTriggerModel = {
    triggerGroups = {} -- array of NodeTriggerGroup
}
function NodeTriggerModel:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end


-- LuaNode
LuaNode = {
    id = "",
    nodename = "",
    children = {},  -- array of string
    effectModel = {}, -- object of NodeEffectElement
    triggerModel = {} -- object of NodeTriggerModel
}
function LuaNode:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- Connection node
ConnectionPair = {
    startNode = "",      -- node id
    startNodePort = "",  -- start group port id
    endNode = ""        -- end node id
}
function ConnectionPair:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

-- luanetwork
LuaNetwork = {
    nodeData = {},  -- array of LuaNode
    connections = {} -- array of ConnectionPair
}
function LuaNetwork:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end



------------------------------------
---    network logic process
------------------------------------
-- Event enum
Event = {
    SetEnable           = 1,
    SetDisable          = 2,
    Stop                = 3
}

local currentPortPool = {}    -- currentPortPool["{PortId}"] = {trigSubtype_1,trigSubtype_2}

local EventQueue = {}
function EventQueue:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self
    return o
end
function EventQueue:append(index, event ,layer)
    if(self[index] == nil) then
        self[index] = {}
    end
    if((self[index][event] == nil)) then
        self[index][event] = {layer}
    else
        table.insert(self[index][event], layer)
    end
end

local eventQueue = EventQueue:new(nil)

local isInPreview = true

function printTab(tab)
    for i,v in pairs(tab) do
        print(i,v)
    end
end

function removeElementByKey(tbl,key)
    --新建一个临时的table
    local tmp ={}

    --把每个key做一个下标，保存到临时的table中，转换成{1=a,2=c,3=b}
    --组成一个有顺序的table，才能在while循环准备时使用#table
    for i in pairs(tbl) do
        table.insert(tmp,i)
    end

    local newTbl = {}
    --使用while循环剔除不需要的元素
    local i = 1
    while i <= #tmp do
        local val = tmp [i]
        if val == key then
            --如果是需要剔除则remove
            table.remove(tmp,i)
         else
            --如果不是剔除，放入新的tabl中
            newTbl[val] = tbl[val]
            i = i + 1
         end
     end
    return newTbl
end

function randomTable(_table, _num)
    local _result = {}
    local _index = 1
    local _num = _num or #_table
    while #_table ~= 0 do
        local ran = math.random(0, #_table)
        if _table[ran] ~= nil then
            _result[_index] = _table[ran]
            table.remove(_table,ran)
            _index = _index + 1
            if _index > _num then
                break
            end
        end
    end
    return _result
end

queueManager = {
    takePhotoBegin = function(this,eventCode,eventMessage)
        print('takePhoto SLREEvent_TAKE_PHOTO_BEGIN')
        if not isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(false)
        networkHandler.initQueue(this,eventMessage)

        isInPreview = false
    end
    ,
    takePhotoDismiss = function(this,eventCode,eventMessage)
        print('takePhoto end')
        if isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(true)
        networkHandler.releaseQueue()

        isInPreview = true
    end
    ,
    videoRecordBegin = function(this, eventCode, eventMessage)
        print('record begin')
        if not isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(false)
        networkHandler.initQueue(this,eventMessage)

        isInPreview = false
    end
    ,
    videoRecordDismiss = function(this, eventCode, eventMessage)
        print('record end')
        if isInPreview then
            return
        end

        initEffects = this:getLayerByName("2DSticker_init")
        initEffects:setEnable(true)
        networkHandler.releaseQueue()

        isInPreview = true
    end
}

networkHandler = {

    initQueue = function(this,eventMessage)
        currentPortPool = {}
        eventQueue = EventQueue:new(nil)

        -- assert(#luaNetwork.nodeData == 0, "Entered An Empty Event")

        ---- find root nodes
        networkHandler.findRootNode(this,eventMessage)
    end
    ,
    releaseQueue = function()
        currentPortPool = nil
        eventQueue = nil
    end
    ,
    findRootNode = function(this,eventMessage)
        local chilrenAssem = {}
        for i,v in pairs(luaNetwork.connections) do
            --print("endnode----------",v.endNode)
            table.insert(chilrenAssem,v.endNode)
        end
        print("chilrenAssem----------")
        printTab(chilrenAssem)
        for i,v in pairs(luaNetwork.nodeData) do
            if (#chilrenAssem == 0) then          ---- no connections
                local node = networkHandler.getNodeByNodeId(v.id)
                effectHandler.setEffectModelParams(this,node,eventMessage)
            elseif (v.nodename == "root") then    ---- circle event
                local node = networkHandler.getNodeByNodeId(v.id)
                effectHandler.setEffectModelParams(this,node,eventMessage)
                for index, value in pairs(v.triggerModel.triggerGroups) do
                    currentPortPool[value.portId] = {}
                end
            else
                print("nodeID------- ",v.id)
                local isAChild = false
                for index, value in pairs(chilrenAssem) do
                    if (v.id == value) then
                        isAChild = true
                        break
                    end
                end
                if(isAChild == false) then
                    print("find root node -----",v.id)
                    local node = networkHandler.getNodeByNodeId(v.id)
                    effectHandler.setEffectModelParams(this,node,eventMessage)
                    for index, value in pairs(v.triggerModel.triggerGroups) do
                        currentPortPool[value.portId] = {}
                    end
                end
            end
        end
    end
    ,
    getNodeByNodeId = function (id)
        -- empty event
        -- assert(luaNetwork.nodeData == nil, "Entered An Empty Event " .. #(luaNetwork.nodeData))
        for i,v in pairs(luaNetwork.nodeData)
        do
            --print("???",v.id,"      ",id)
            if(v.id == id) then
                return v
            end
        end
        assert(true,"Unknown NodeId")
    end
    ,
    getNodeByPortId = function(id)
        for key,value in pairs(luaNetwork.nodeData)
        do
            for k,v in pairs(value.triggerModel.triggerGroups)
            do
                if(v.portId == id) then
                    return value
                end
            end
        end
        assert(true,"Unknown PortId")
    end
    ,
    getChildNodebyPortId = function (id)
        print("connections------- ",#luaNetwork.connections)
        for key, value in pairs(luaNetwork.connections) do
            print("id-----------",id,"     ",value.startNodePort)
            if(id == value.startNodePort) then
                return networkHandler.getNodeByNodeId(value.endNode)
            end
        end
    end
    ,
    getPortsByNodeId = function (id)
        local ports = {}
        local node =  networkHandler.getNodeByNodeId(id)
        --print("node----------  ",node)
        -- leaf node
        local triggerGroups = node.triggerModel.triggerGroups
        print("triggerGroups------ ",#triggerGroups)
        if(#triggerGroups == 0) then
            return ports
        end
        for i,v in pairs(triggerGroups)
        do
            table.insert(ports,v.portId)
        end

        return ports
    end
    ,
    getChildPortsByPortId = function (id)
        local endNode = ""
        --print("1 ---------", #luaNetwork.connections)
        --print("2---------------",id)
        -- no connections
        if(#luaNetwork.connections == 0) then
            return {}
        end

        for i,v in pairs(luaNetwork.connections)
        do
            --print("3-------------",id,'     ',v.startNodePort)
            if(v.startNodePort == id)   then
                endNode = v.endNode
                break
            end
        end
        --print("4-------------",endNode)
        return networkHandler.getPortsByNodeId(endNode)
    end
    ,
    getTriggerGroupByPortId = function (id)
        -- cannot use connection search for leaf ports
        -- port already exsists
        for key,value in pairs(luaNetwork.nodeData)
        do
            for k,v in pairs(value.triggerModel.triggerGroups)
            do
                if(v.portId == id) then
                    return v
                end
            end
        end
        assert(true,"Unknown PortId")
    end

}
local flfag = true
effectHandler = {
    newframeHandle = function(this,eventMessage)            ----  set subLayer enable / disable / stop
        local ensuranceCoefficient = 5                     ---- process specific lost frame event
        local frameId = eventMessage.frameId

        if(flfag) then
            networkHandler.initQueue(this,eventMessage)
            flfag = false
        end

        --print("eventQueue-----------")
        --printTab(eventQueue)
        --print("eventQueue-----------")
        --print("currentPortPool----------")
        --printTab(currentPortPool)
        --print("currentPortPool----------")
        --print("frameId ---- ",frameId)

        effectHandler.processFrameEvent(frameId,ensuranceCoefficient)

        -- if(frameId == 200 ) then
        --     EventHandles.handleEffectEvent(this,202, eventMessage)
        -- end

    end
    ,
    processFrameEvent = function(frameId,ensuranceCoefficient)
        local coefficient = 0
        if (frameId > ensuranceCoefficient) then
            coefficient = ensuranceCoefficient
        else
            coefficient = frameId - 1
        end
        for i = coefficient,0,-1 do
            if (eventQueue[frameId - i] ~= nil) then

                if (eventQueue[frameId - i][Event.SetDisable]) then
                    for i,v in pairs(eventQueue[frameId - i][Event.SetDisable]) do
                        v:setEnable(false)
                    end
                end
                if (eventQueue[frameId - i][Event.SetEnable]) then
                    print("setenable--------   ",#eventQueue[frameId - i][Event.SetEnable])
                    for i,v in pairs(eventQueue[frameId - i][Event.SetEnable]) do
                        v:setEnable(true)
                    end
                end
                if (eventQueue[frameId - i][Event.Stop]) then
                    for i,v in pairs(eventQueue[frameId - i][Event.Stop]) do
                        v:setClipAutoRepeat(false)
                    end
                end
                eventQueue[frameId - i] = nil
            end
        end
    end
    ,
    setEffectModelParams = function(this,node,eventMessage)     -- set layer peoperties and add to queues
        if(node ~= nil) then
            local effectModel = node.effectModel
            local frameId = 1
            if(eventMessage) then
                frameId = eventMessage.frameId
            end

            if(#effectModel.effectElements > 0) then
                if(effectModel.playOrder == PlayOrder.Simultaneously) then
                    effectHandler.effectModelSimultaneouslyPlay(this,effectModel.effectElements,frameId)
                elseif(effectModel.playOrder == PlayOrder.Sequentially) then
                    effectHandler.effectModelSequentiallyPlay(this,effectModel.effectElements,frameId)
                elseif(effectModel.playOrder == PlayOrder.Randomly) then
                    effectHandler.effectModelRandomlyPlay(this,effectModel.effectElements,frameId)
                else
                    effectHandler.effectModelRandomlyPlayOnce(this,effectModel.effectElements,frameId)
                end
            end
        end
    end
    ,

    -- set params of facesticker/fg effect element
    setStickerEffect = function(effectElement,subLayer,frameId)
        local duration = 0
        if(effectElement.animationAction == AnimationAction.Appear) then
            subLayer:setClipEpochFrame(frameId)
            --subLayer:setClipPlayRange(effectElement.startFrame,effectElement.endFrame)

            --setvalidPeriod
            subLayer:setClipAutoRepeat(true)
            eventQueue:append(frameId,Event.SetEnable,subLayer)

            local fps_video = subLayer:getVideoFPS() --subLayer.getClipFrameCount()
            local fps_output = subLayer:getAssetFPS()--subLayer.getRangeClipFrameCount()
            local frameCounts = subLayer:getClipFrameCount()
            print("fps_output-------  ",fps_output)
            print("fps_video-------  ",fps_video)
            print("frameCounts-------  ",frameCounts)
            print("effectElement.validPeriodParam-------  ",effectElement.validPeriodParam)
            if(effectElement.validPeriod == ValidPeriod.Indefinite) then
                --do nothing
                duration = nil
            elseif(effectElement.validPeriod == ValidPeriod.DefineTime) then

                duration = fps_video * effectElement.validPeriodParam
                eventQueue:append(frameId + duration, Event.SetDisable, subLayer)
            else
                duration = effectElement.validPeriodParam * frameCounts  --(frameCounts + 1)* (fps_video / fps_output)
                if(effectElement.stopAtLastFrame) then
                    eventQueue:append(frameId + duration , Event.Stop, subLayer)

                    if(effectElement.keepForever) then
                        --only need to set unrepeative
                    else
                        --need to be set disabled
                        eventQueue:append(frameId + duration + effectElement.keepDuration * fps_video, Event.SetDisable, subLayer)
                    end
                else
                    eventQueue:append(frameId + duration, Event.SetDisable, subLayer)
                end
            end
        else
            eventQueue:append(frameId, Event.SetDisable, subLayer)
        end
        return duration
    end
    ,
    -- set params of filter effect element
    setLutEffect = function(effectElement,subLayer,frameId)
        print("sublayer-------    ",subLayer)

        if (effectElement.animationAction == AnimationAction.Appear) then
            eventQueue:append(frameId,Event.SetEnable,subLayer)
            --eventQueue:append(frameId,Event.SetEnable,subLayer)
        else
            eventQueue:append(frameId,Event.SetDisable,subLayer)
            --eventQueue:append(frameId,Event.SetDisable,subLayer)
        end
    end
    ,
    effectModelSimultaneouslyPlay = function(this,effectElements,frameId)
        for i,v in pairs(effectElements) do
            print("stickername-----",v.name)
            print("layers--------",#this:getLayers())
            for j,t in pairs(this:getLayers()) do
                print("layers--------",t:getName())
                --print("sublayers--------",t:getLayers()[1]:getName())
            end
            local curSubLayer = this:getLayerByName(v.name)
            print("curSubLayer--------",curSubLayer)
            if(v.type == "FaceStickerElement" or v.type == "ForeStickerElement" or v.type == "CatStickerElement") then
                effectHandler.setStickerEffect(v,curSubLayer,frameId)
            else
                effectHandler.setLutEffect(v,curSubLayer,frameId)
            end
        end
    end
    ,
    effectModelSequentiallyPlay = function(this,effectElements,frameId)

        local playFrames = 0
        local duration = 0
        for i, v in pairs(effectElements) do
            local curSubLayer = this:getLayerByName(v.name)
            if(v.type ~= "FilterEffectElement") then
                duration = effectHandler.setStickerEffect(v,curSubLayer,frameId)
                if(duration) then
                    frameId = frameId + duration
                end
            else
                effectHandler.setLutEffect(v,curSubLayer,frameId)
            end
        end
    end
    ,
    effectModelRandomlyPlay = function(this,effectElements,frameId)
        -- create a random table
        local rdmTbl = {}
        local playFrames = 0
        for i = 1,#effectElements do
            table.insert(rdmTbl,i)
        end
        rdmTbl = randomTable(rdmTbl)
        local duration = 0
        for i,v in pairs(rdmTbl) do
            local curSubLayer = this:getLayerByName(effectElements[v].name)
            if(effectElements[v].type ~= "FilterEffectElement") then
                duration = effectHandler.setStickerEffect(effectElements[v],curSubLayer,frameId)
                if(duration) then
                    frameId = frameId + duration
                end
            else
                effectHandler.setLutEffect(effectElements[v],curSubLayer,frameId)
            end
        end
    end
    ,
    effectModelRandomlyPlayOnce = function (this,effectElements,frameId)
        local pick = math.random(#effectElements)
        local curSubLayer = this:getLayerByName(effectElements[pick].name)

        if(effectElements[pick].type ~= "FilterEffectElement") then
            effectHandler.setStickerEffect(effectElements[pick],curSubLayer,frameId)
        else
            effectHandler.setLutEffect(effectElements[pick],curSubLayer,frameId)
        end

    end
}
triggerhandler = {

    nodeTrigged = function (this,portId,eventMessage)
        -- 1. delete current port
        currentPortPool = removeElementByKey(currentPortPool,portId)
        -- 2.set effects params
        local node = networkHandler.getChildNodebyPortId(portId)

        effectHandler.setEffectModelParams(this,node,eventMessage)
        -- 3.insert child ports
        local childPorts = networkHandler.getChildPortsByPortId(portId)
        print("childports--------   ",#childPorts)

        if(#childPorts > 0)
        then
            for i,v in pairs(childPorts)
            do
                currentPortPool[v] = {}
            end
        end

        -- leaf port needs no hanle

    end
    ,
    updatePortPool = function(portId,eventCode)
        -- add unique evenetcode to table
        local flag = true

        for i, v in pairs(currentPortPool[portId])
        do
            if(v == eventCode)
            then
                flag = false
                break
            end
        end

        if(flag)
        then
            table.insert(currentPortPool[portId],eventCode)
        end
    end
    ,
    handleFaceTrigger = function(this, trigSubType,eventMessage)
        for key, value in pairs(currentPortPool)
        do
            local triggerGroup = networkHandler.getTriggerGroupByPortId(key)       --  a triggerGroup is not empty by default
            local trigCondition = triggerGroup.trigCondition
            local triggerElements = triggerGroup.triggerElements
            local portId = key
            for i, v in pairs(triggerElements)
            do
                if(v.trigType == TrigType.Face and v.trigSubType == trigSubType)
                then
                    if(trigCondition == TrigCondition.Or)
                    then
                        triggerhandler.nodeTrigged(this,portId,eventMessage)
                    else
                        triggerhandler.updatePortPool(portId,trigSubType)
                        if(#currentPortPool[portId] == #triggerElements) then
                            triggerhandler.nodeTrigged(this,portId,eventMessage)
                        end
                    end

                end
            end
        end
    end
    ,
    handleHandTrigger = function(this, trigSubType,eventMessage)
        for key, value in pairs(currentPortPool)
        do
            local triggerGroup = networkHandler.getTriggerGroupByPortId(key)       --  a triggerGroup is not empty by default
            local trigCondition = triggerGroup.trigCondition
            local triggerElements = triggerGroup.triggerElements
            local portId = key
            for index, v in pairs(triggerElements)
            do
                if(v.trigType == TrigType.Hand and v.trigSubType == trigSubType) then
                    if(trigCondition == TrigCondition.Or) then
                        triggerhandler.nodeTrigged(this,portId,eventMessage)
                    else
                        triggerhandler.updatePortPool(portId,trigSubType)
                        if(#currentPortPool[portId] == #triggerElements) then
                            triggerhandler.nodeTrigged(this,portId,eventMessage)
                        end
                    end

                end
            end
        end
    end
}

------------------------------------
---    event entry
------------------------------------
EventHandleHelper = {

}
EventHandles = {
    -- this is one effect item
    handleTouchEvent = function (this, eventCode, eventMessage)
        if (eventCode == SLRESdk.EventCode.SLREEvent_TOUCHUP_INSIDE) then
            -- print('lua: touch event')
        end
        return true
    end,
    handleEffectEvent = function (this, eventCode, eventMessage)
        --print("事件-------  ",eventCode)
        if eventCode >= SLRESdk.EventCode.SLREEvent_FACE_DETECT and eventCode <= SLRESdk.EventCode.SLREEvent_EXPRESSION_MAX then
            trigType = TrigType.Face
            trigSubType = eventCode - SLRESdk.EventCode.SLREEvent_FACE_DETECT
            triggerhandler.handleFaceTrigger(this,trigSubType,eventMessage)
        elseif eventCode >= SLRESdk.EventCode.SLREEvent_HAND_APPEAR and eventCode < SLRESdk.EventCode.SLREEvent_HAND_MAX then
            trigType = TrigType.Hand
            trigSubType = eventCode - SLRESdk.EventCode.SLREEvent_HAND_APPEAR
            triggerhandler.handleHandTrigger(this, trigSubType,eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_NEW_FRAME then
            effectHandler.newframeHandle(this, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_TAKE_PHOTO_BEGIN then
            queueManager.takePhotoBegin(this, eventCode, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_TAKE_PHOTO_DIMISS
            or eventCode == SLRESdk.EventCode.SLREEvent_TAKE_PHOTO_END then
            queueManager.takePhotoDismiss(this, eventCode, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_VIDEO_RECORD_BEGIN then
            queueManager.videoRecordBegin(this, eventCode, eventMessage)
        elseif eventCode == SLRESdk.EventCode.SLREEvent_VIDEO_RECORD_DIMISS
            or eventCode == SLRESdk.EventCode.SLREEvent_VIDEO_RECORD_END then
            queueManager.videoRecordDismiss(this, eventCode, eventMessage)
        end
        return true
    end,
    handQueryEvent = function (this, eventCode, eventMessage)
        if eventCode == SLRESdk.EventCode.SLREEvent_QUERY_NEED_ANI_PHOTO_TAKE then
            return 0
        end
        return 0
    end,
}

------------------------------------
---    build netowrk
------------------------------------




    function buildLuaNetwork()
        local luaNetwork = LuaNetwork:new()
        luaNetwork.repetitive = true
        luaNetwork.resetEffect = true
        luaNetwork.nodeData = 
        (function ()
            local nodes = {}
                            
                local nodeElement =     
                (function ()
                    local luaNode = LuaNode:new()
                    luaNode.id = "{5cacbb85-6c0d-47be-a842-cb893b6fb760}"
                    luaNode.nodename = "节点1"
                    luaNode.children = {}
                    luaNode.effectModel =     
                    (function ()
                        local effectModel = NodeEffectModel:new()
                        
                        effectModel.eventType = 1
                        effectModel.playOrder = 0
                        
                        effectModel.effectElements = 
                        (function ()
                            local effectElements = {}
                                                
                            local effectElement =         
                            (function ()
                                local effectElement = NodeEffectElement:new()
                                
                                effectElement.type = "FaceStickerElement"
                                effectElement.name = "default_FaceSticker_2"
                                
                                effectElement.animationAction = 1
                                
                                effectElement.startFrame = 1
                                effectElement.endFrame = 1
                                effectElement.validPeriod = 0
                                effectElement.validPeriodParam = 1
                        
                                effectElement.stopAtLastFrame = false
                                effectElement.keepForever = true
                                effectElement.keepDuration = 1
                                
                                return effectElement
                            end)()
                                            
                            table.insert(effectElements,effectElement)
                                                
                            local effectElement =         
                            (function ()
                                local effectElement = NodeEffectElement:new()
                                
                                effectElement.type = "FaceStickerElement"
                                effectElement.name = "default_FaceSticker_1"
                                
                                effectElement.animationAction = 0
                                
                                effectElement.startFrame = 1
                                effectElement.endFrame = 1
                                effectElement.validPeriod = 0
                                effectElement.validPeriodParam = 1
                        
                                effectElement.stopAtLastFrame = false
                                effectElement.keepForever = true
                                effectElement.keepDuration = 1
                                
                                return effectElement
                            end)()
                                            
                            table.insert(effectElements,effectElement)
                                                
                            local effectElement =         
                            (function ()
                                local effectElement = NodeEffectElement:new()
                                
                                effectElement.type = "FaceStickerElement"
                                effectElement.name = "default_FaceSticker_0"
                                
                                effectElement.animationAction = 0
                                
                                effectElement.startFrame = 1
                                effectElement.endFrame = 1
                                effectElement.validPeriod = 0
                                effectElement.validPeriodParam = 1
                        
                                effectElement.stopAtLastFrame = false
                                effectElement.keepForever = true
                                effectElement.keepDuration = 1
                                
                                return effectElement
                            end)()
                                            
                            table.insert(effectElements,effectElement)
                            
                            return effectElements
                        end)()
                
                        
                        return effectModel
                    end)()
                
                    luaNode.triggerModel =     
                    (function ()
                        local triggerModel = NodeTriggerModel:new()
                        
                        triggerModel.triggerGroups = 
                        (function ()
                            local triggerGroups = {}
                                                
                            local triggerGroup =         
                                (function ()
                                    local triggerGroup = NodeTriggerGroup:new()
                                    
                                    triggerGroup.eventType = 2
                                    triggerGroup.portId = "{e04a3913-91ba-494e-b64c-7ef944e8928d}"
                                    triggerGroup.trigCondition = 0
                                    triggerGroup.triggerElements = 
                                    (function ()
                                        local triggerElements = {}
                                                                
                                        local triggerElement =             
                                        (function ()
                                            local triggerElement = NodeTriggerElement:new()
                                            
                                            triggerElement.name = ""
                                            triggerElement.trigType = 2
                                            triggerElement.trigSubType = 31
                                            
                                            return triggerElement
                                        end)()
                                                            
                                        table.insert(triggerElements,triggerElement)
                                        
                                        return triggerElements
                                    end)()
                        
                                    
                                    return triggerGroup
                                end)()
                                            
                            table.insert(triggerGroups,triggerGroup)
                            
                            return triggerGroups
                        end)()
                
                
                        return triggerModel
                    end)()
                
                    return luaNode
                end)()
                            
                table.insert(nodes,nodeElement)
                                
                local nodeElement =     
                (function ()
                    local luaNode = LuaNode:new()
                    luaNode.id = "{d02b4591-7f55-4fc3-a8a6-c039e2335e89}"
                    luaNode.nodename = "root"
                    luaNode.children = {}
                    luaNode.effectModel =     
                    (function ()
                        local effectModel = NodeEffectModel:new()
                        
                        effectModel.eventType = 1
                        effectModel.playOrder = 0
                        
                        effectModel.effectElements = 
                        (function ()
                            local effectElements = {}
                                                
                            local effectElement =         
                            (function ()
                                local effectElement = NodeEffectElement:new()
                                
                                effectElement.type = "FaceStickerElement"
                                effectElement.name = "default_FaceSticker_0"
                                
                                effectElement.animationAction = 1
                                
                                effectElement.startFrame = 1
                                effectElement.endFrame = 1
                                effectElement.validPeriod = 0
                                effectElement.validPeriodParam = 1
                        
                                effectElement.stopAtLastFrame = false
                                effectElement.keepForever = true
                                effectElement.keepDuration = 1
                                
                                return effectElement
                            end)()
                                            
                            table.insert(effectElements,effectElement)
                                                
                            local effectElement =         
                            (function ()
                                local effectElement = NodeEffectElement:new()
                                
                                effectElement.type = "FaceStickerElement"
                                effectElement.name = "default_FaceSticker_1"
                                
                                effectElement.animationAction = 1
                                
                                effectElement.startFrame = 1
                                effectElement.endFrame = 1
                                effectElement.validPeriod = 0
                                effectElement.validPeriodParam = 1
                        
                                effectElement.stopAtLastFrame = false
                                effectElement.keepForever = true
                                effectElement.keepDuration = 1
                                
                                return effectElement
                            end)()
                                            
                            table.insert(effectElements,effectElement)
                                                
                            local effectElement =         
                            (function ()
                                local effectElement = NodeEffectElement:new()
                                
                                effectElement.type = "FaceStickerElement"
                                effectElement.name = "default_FaceSticker_2"
                                
                                effectElement.animationAction = 0
                                
                                effectElement.startFrame = 1
                                effectElement.endFrame = 1
                                effectElement.validPeriod = 0
                                effectElement.validPeriodParam = 1
                        
                                effectElement.stopAtLastFrame = false
                                effectElement.keepForever = true
                                effectElement.keepDuration = 1
                                
                                return effectElement
                            end)()
                                            
                            table.insert(effectElements,effectElement)
                            
                            return effectElements
                        end)()
                
                        
                        return effectModel
                    end)()
                
                    luaNode.triggerModel =     
                    (function ()
                        local triggerModel = NodeTriggerModel:new()
                        
                        triggerModel.triggerGroups = 
                        (function ()
                            local triggerGroups = {}
                                                
                            local triggerGroup =         
                                (function ()
                                    local triggerGroup = NodeTriggerGroup:new()
                                    
                                    triggerGroup.eventType = 2
                                    triggerGroup.portId = "{889ae18b-c1e2-4c26-b1df-550bd847ae3f}"
                                    triggerGroup.trigCondition = 0
                                    triggerGroup.triggerElements = 
                                    (function ()
                                        local triggerElements = {}
                                                                
                                        local triggerElement =             
                                        (function ()
                                            local triggerElement = NodeTriggerElement:new()
                                            
                                            triggerElement.name = ""
                                            triggerElement.trigType = 2
                                            triggerElement.trigSubType = 23
                                            
                                            return triggerElement
                                        end)()
                                                            
                                        table.insert(triggerElements,triggerElement)
                                        
                                        return triggerElements
                                    end)()
                        
                                    
                                    return triggerGroup
                                end)()
                                            
                            table.insert(triggerGroups,triggerGroup)
                            
                            return triggerGroups
                        end)()
                
                
                        return triggerModel
                    end)()
                
                    return luaNode
                end)()
                            
                table.insert(nodes,nodeElement)
                
            return nodes
        end)()
    
        luaNetwork.connections = 
        (function ()
            local connections = {}
                            
                local connectionElement =     
                (function ()
                    local connectionPair = ConnectionPair:new()
                    connectionPair.startNode = "{5cacbb85-6c0d-47be-a842-cb893b6fb760}"
                    connectionPair.startNodePort = "{e04a3913-91ba-494e-b64c-7ef944e8928d}"
                    connectionPair.endNode = "{d02b4591-7f55-4fc3-a8a6-c039e2335e89}"
                    return connectionPair
                end)()
                            
                table.insert(connections,connectionElement)
                                
                local connectionElement =     
                (function ()
                    local connectionPair = ConnectionPair:new()
                    connectionPair.startNode = "{d02b4591-7f55-4fc3-a8a6-c039e2335e89}"
                    connectionPair.startNodePort = "{889ae18b-c1e2-4c26-b1df-550bd847ae3f}"
                    connectionPair.endNode = "{5cacbb85-6c0d-47be-a842-cb893b6fb760}"
                    return connectionPair
                end)()
                            
                table.insert(connections,connectionElement)
                
            return connections
        end)()
    
        return luaNetwork
    end
    luaNetwork = buildLuaNetwork()
    